@@ -82,6 +82,11 @@ class RasterCompare:
              fn=lambda v1, v2: len(v1))
     ]
 
+    @property
+    def proc_crs(self) -> ProcCrs:
+        """The 'processing CRS' i.e. which of the source/reference image spaces is selected for processing."""
+        return self._proc_crs
+
     @property
     def stats_key(self):
         """Returns a string of abbreviations and corresponding descriptions for the statistics returned by compare()"""
@@ -97,10 +102,16 @@ class RasterCompare:
                   A dictionary representing the results.
         """
         res_dict = OrderedDict()
+
         bar_format = '{l_bar}{bar}|{n_fmt}/{total_fmt} bands [{elapsed}<{remaining}]'
 
         with self._raster_pair as raster_pair:
 
+            def get_res_key(band_i):
+                return (raster_pair.ref_im.descriptions[raster_pair.ref_bands[band_i] - 1] or
+                               raster_pair.src_im.descriptions[raster_pair.src_bands[band_i] - 1] or
+                               f'Band {band_i + 1}')
+
             def process_band(block_pair):
                 """Thread-safe function to process a block (that encompasses the full band)"""
                 src_ra, ref_ra = raster_pair.read(block_pair)  # read src and ref bands
@@ -122,11 +133,14 @@ class RasterCompare:
                     stats_dict[_stat['ABBREV']] = _stat['fn'](src_vec, ref_vec)
 
                 # form a string describing the band
-                description = (raster_pair.ref_im.descriptions[raster_pair.ref_bands[block_pair.band_i] - 1] or
-                        raster_pair.src_im.descriptions[raster_pair.src_bands[block_pair.band_i] - 1] or
-                        f'Band {block_pair.band_i + 1}')
+                description = get_res_key(block_pair.band_i)
                 return description, stats_dict
 
+            # populate res_dict with band-ordered keys
+            for band_i in range(len(raster_pair.src_bands)):
+                description = get_res_key(band_i)
+                res_dict[description] = None
+
             # process bands concurrently
             with concurrent.futures.ThreadPoolExecutor(max_workers=self._threads) as executor:
                 futures = [executor.submit(process_band, block_pair) for block_pair in raster_pair.block_pairs()]
