@@ -53,7 +53,7 @@ class RasterFuse:
     default_model_config = KernelModel.default_config
 
     def __init__(self, src_filename, ref_filename, homo_path, method, kernel_shape, proc_crs=ProcCrs.auto,
-                 overwrite=True, homo_config=None, model_config=None, out_profile=None):
+                 overwrite=False, homo_config=None, model_config=None, out_profile=None):
         """
         Create a RasterFuse class.
 
@@ -119,7 +119,6 @@ class RasterFuse:
         self._method = method
 
         self._kernel_shape = utils.validate_kernel_shape(kernel_shape, method=method)
-        homo_config['threads'] = utils.validate_threads(homo_config['threads'])
 
         self._src_filename = pathlib.Path(src_filename)
         self._ref_filename = pathlib.Path(ref_filename)
@@ -127,6 +126,8 @@ class RasterFuse:
         self._model_config = model_config if model_config else self.default_model_config
         self._out_profile = out_profile if out_profile else self.default_out_profile
 
+        self._config['threads'] = utils.validate_threads(self._config['threads'])
+
         # get the block overlap for kernel_shape
         overlap = utils.overlap_for_kernel(self._kernel_shape)
 
@@ -140,12 +141,10 @@ class RasterFuse:
         # create the KernelModel according to proc_crs
         if self._proc_crs == ProcCrs.ref:
             self._model = RefSpaceModel(method=method, kernel_shape=kernel_shape,
-                                        param_image=self._config['param_image'], **model_config)
+                                        param_image=self._config['param_image'], **self._model_config)
         elif self._proc_crs == ProcCrs.src:
             self._model = SrcSpaceModel(method=method, kernel_shape=kernel_shape,
-                                        param_image=self._config['param_image'], **model_config)
-        else:
-            raise ValueError(f'Invalid proc_crs: {proc_crs}, should be resolved to ProcCrs.ref or ProcCrs.src')
+                                        param_image=self._config['param_image'], **self._model_config)
 
         # initialise other variables
         self._write_lock = threading.Lock()
@@ -174,32 +173,11 @@ class RasterFuse:
             raise FileExistsError(f"Parameter image file exists and won't be overwritten without the "
                                   f"'overwrite' option: {self._param_filename}")
 
-    def _combine_with_config(self, in_profile):
-        """Update an input rasterio profile with the configuration profile."""
-
-        if in_profile['driver'].lower() != self._out_profile['driver'].lower():
-            # copy only non driver specific keys from input profile when the driver is different to the configured val
-            copy_keys = ['driver', 'width', 'height', 'count', 'dtype', 'crs', 'transform']
-            out_profile = {copy_key: in_profile[copy_key] for copy_key in copy_keys}
-        else:
-            out_profile = in_profile.copy()  # copy the whole input profile
-
-        def nested_update(self_dict, other_dict):
-            """Update self_dict with a flattened version of other_dict"""
-            for other_key, other_value in other_dict.items():
-                if isinstance(other_value, dict):
-                    # flatten the driver specific nested dict into the root dict
-                    nested_update(self_dict, other_value)
-                else:
-                    self_dict[other_key] = other_value
-            return self_dict
-
-        # update out_profile with a flattened self._out_profile
-        return nested_update(out_profile, self._out_profile)
 
     def _create_out_profile(self) -> dict:
         """Create an output image rasterio profile from the source image profile and output configuration"""
-        out_profile = self._combine_with_config(self._raster_pair.src_im.profile)
+        # out_profile = self._combine_with_config(self._raster_pair.src_im.profile)
+        out_profile = utils.combine_profiles(self._raster_pair.src_im.profile, self._out_profile)
         out_profile['count'] = len(self._raster_pair.src_bands)
         return out_profile
 
@@ -209,7 +187,8 @@ class RasterFuse:
             init_profile = self._raster_pair.ref_im.profile
         else:
             init_profile = self._raster_pair.src_im.profile
-        param_profile = self._combine_with_config(init_profile)
+        # param_profile = self._combine_with_config(init_profile)
+        param_profile = utils.combine_profiles(init_profile, self._out_profile)
         # force dtype and nodata to defaults
         param_profile.update(dtype=RasterArray.default_dtype, count=len(self._raster_pair.src_bands) * 3,
                              nodata=RasterArray.default_nodata)
@@ -267,7 +246,7 @@ class RasterFuse:
     def _assert_open(self):
         """Raise an IoError if the source, reference or homogenised image(s) are not open."""
         if self.closed:
-            raise IoError(f'The raster file(s) have not been opened.')
+            raise IoError(f'The image file(s) have not been opened.')
 
     def open(self):
         """Open the source and reference images for reading, and output image(s) for writing."""
@@ -327,32 +306,42 @@ class RasterFuse:
         """Path to the parameter image file."""
         return self._param_filename if self._config['param_image'] else None
 
-    def _build_overviews(self, im):
+    def _build_overviews(self, im, max_num_levels=8, min_level_pixels=256):
         """
-        Build internal overviews for a rasterio dataset.
+        Build internal overviews, downsampled by successive powers of 2, for a rasterio dataset.
 
         Parameters
         ----------
         im: rasterio.io.DatasetWriter
             An open rasterio dataset to write the metadata to.
+        max_num_levels: int, optional
+            Maximum number of overview levels to build.
+        min_level_pixels: int, pixel
+            Minimum width/height (in pixels) of any overview level.
         """
 
         # limit overviews so that the highest level has at least 2**8=256 pixels along the shortest dimension,
         # and so there are no more than 8 levels.
         max_ovw_levels = int(np.min(np.log2(im.shape)))
-        num_ovw_levels = np.min([8, max_ovw_levels - 8])
+        min_level_shape_pow2 = int(np.log2(min_level_pixels))
+        num_ovw_levels = np.min([max_num_levels, max_ovw_levels - min_level_shape_pow2])
         ovw_levels = [2 ** m for m in range(1, num_ovw_levels + 1)]
         im.build_overviews(ovw_levels, Resampling.average)
 
-    def build_overviews(self):
+    def build_overviews(self, max_num_levels=8, min_level_pixels=256):
         """
-        Build overviews for the homogenised and parameter image files.
+        Build internal overviews, downsampled by successive powers of 2, for the homogenised and parameter image files.
         Should be called after homogenise().
+
+        max_num_levels: int, optional
+            Maximum number of overview levels to build.
+        min_level_pixels: int, pixel
+            Minimum width/height (in pixels) of any overview level.
         """
         self._assert_open()
-        self._build_overviews(self._out_im)
+        self._build_overviews(self._out_im, max_num_levels=max_num_levels, min_level_pixels=min_level_pixels)
         if self._config['param_image']:
-            self._build_overviews(self._param_im)
+            self._build_overviews(self._param_im, max_num_levels=max_num_levels, min_level_pixels=min_level_pixels)
 
     def _process_block(self, block_pair: BlockPair):
         """Thread-safe function to homogenise a source image block"""
