@@ -19,7 +19,6 @@
 
 from typing import Tuple
 
-import cv2
 import cv2 as cv
 import numpy as np
 from homonim import utils
@@ -105,7 +104,11 @@ class KernelModel:
     def config(self) -> dict:
         """A dict containing the KernelModel configuration."""
         return dict(r2_inpaint_thresh=self._r2_inpaint_thresh, downsampling=self._downsampling,
-                    upsampling=self._upsampling)
+                    upsampling=self._upsampling, mask_partial=self._mask_partial)
+
+    def _get_resampling(self, from_res, to_res):
+        """Get the resampling method for re-projecting from resolution `from_res` to resolution `to_res`"""
+        return self._downsampling if np.prod(np.abs(from_res)) <= np.prod(np.abs(to_res)) else self._upsampling
 
     def _r2_array(self, ref_array, src_array, param_array, mask=None, mask_sum=None, ref_sum=None, src_sum=None,
                   ref2_sum=None, src2_sum=None, src_ref_sum=None, dest_array=None, kernel_shape=None):
@@ -121,34 +124,34 @@ class KernelModel:
         # find the keyword arguments that were not provided
         if mask is None:
             # if mask is passed, assume that it has been applied ref_array and src_array, otherwise do that here
-            mask = (utils.nan_equals(src_array, RasterArray.default_nodata) &
-                    utils.nan_equals(ref_array, RasterArray.default_nodata))
+            mask = (~utils.nan_equals(src_array, RasterArray.default_nodata) &
+                    ~utils.nan_equals(ref_array, RasterArray.default_nodata))
             ref_array[~mask] = 0
             src_array[~mask] = 0
         if mask_sum is None:
-            mask_sum = cv.boxFilter(mask.astype(RasterArray.default_dtype), -1, kernel_shape, **filter_args)
+            mask_sum = cv.boxFilter(mask.astype(RasterArray.default_dtype), -1, kernel_shape[::-1], **filter_args)
         if ref_sum is None:
-            ref_sum = cv.boxFilter(ref_array, -1, kernel_shape, **filter_args)
+            ref_sum = cv.boxFilter(ref_array, -1, kernel_shape[::-1], **filter_args)
         if ref2_sum is None:
-            ref2_sum = cv.sqrBoxFilter(ref_array, -1, kernel_shape, **filter_args)
+            ref2_sum = cv.sqrBoxFilter(ref_array, -1, kernel_shape[::-1], **filter_args)
         if src2_sum is None:
-            src2_sum = cv.sqrBoxFilter(src_array, -1, kernel_shape, **filter_args)
+            src2_sum = cv.sqrBoxFilter(src_array, -1, kernel_shape[::-1], **filter_args)
         if src_ref_sum is None:
-            src_ref_sum = cv.boxFilter(src_array * ref_array, -1, kernel_shape, **filter_args)
+            src_ref_sum = cv.boxFilter(src_array * ref_array, -1, kernel_shape[::-1], **filter_args)
 
         # R2 is found using: R2 = 1 - (residual sum of squares)/(total sum of squares) = 1 - RSS/TSS
 
-        # TSS = sum((ref - mean(ref))**2), which can be expanded and expressed in terms of cv2.boxFilter kernel sums as:
+        # TSS = sum((ref - mean(ref))**2), which can be expanded and expressed in terms of cv.boxFilter kernel sums as:
         ss_tot_array = (mask_sum * ref2_sum) - (ref_sum ** 2)
 
         if param_array.shape[0] > 1:
             # find RSS for method == Method.gain_offset
             if src_sum is None:
-                src_sum = cv.boxFilter(src_array, -1, kernel_shape, **filter_args)
+                src_sum = cv.boxFilter(src_array, -1, kernel_shape[::-1], **filter_args)
 
             # RSS = sum((ref - ref_hat)**2)
             #     = sum((ref - (m*src + c))**2), where m and c are the first 2 bands of param_array
-            # The above can be expanded and expressed in terms of cv2.boxFilter kernel sums as:
+            # The above can be expanded and expressed in terms of cv.boxFilter kernel sums as:
 
             ss_res_array = (((param_array[0] ** 2) * src2_sum) +
                             (2 * np.product(param_array[:2], axis=0) * src_sum) -
@@ -158,7 +161,7 @@ class KernelModel:
         else:
             # find RSS for method == Method.gain or Method.gain_blk_offset
             # RSS = sum((ref - m*src)**2), where m is the first band of param_array
-            # The above can be expanded and expressed in terms of cv2.boxFilter kernel sums as:
+            # The above can be expanded and expressed in terms of cv.boxFilter kernel sums as:
             ss_res_array = (((param_array[0] ** 2) * src2_sum) - (2 * param_array[0] * src_ref_sum) + ref2_sum)
 
         ss_res_array *= mask_sum
@@ -237,8 +240,8 @@ class KernelModel:
 
         # convolve the kernel with src and ref to get kernel sums (uses DFT for large kernels)
         filter_args = dict(normalize=False, borderType=cv.BORDER_CONSTANT)  # common opencv arguments
-        src_sum = cv.boxFilter(src_array, -1, kernel_shape, **filter_args)
-        ref_sum = cv.boxFilter(ref_array, -1, kernel_shape, **filter_args)
+        src_sum = cv.boxFilter(src_array, -1, kernel_shape[::-1], **filter_args)
+        ref_sum = cv.boxFilter(ref_array, -1, kernel_shape[::-1], **filter_args)
 
         # create parameter RasterArray filled with nodata
         param_ra = RasterArray.from_profile(None, param_profile)
@@ -329,14 +332,14 @@ class KernelModel:
 
         # find the numerator for the gain i.e N*cov(ref, src)
         filter_args = dict(normalize=False, borderType=cv.BORDER_CONSTANT)  # common opencv arguments
-        src_sum = cv.boxFilter(src_array, -1, kernel_shape, **filter_args)
-        ref_sum = cv.boxFilter(ref_array, -1, kernel_shape, **filter_args)
-        src_ref_sum = cv.boxFilter(src_array * ref_array, -1, kernel_shape, **filter_args)
-        mask_sum = cv.boxFilter(mask.astype(RasterArray.default_dtype, copy=False), -1, kernel_shape, **filter_args)
+        src_sum = cv.boxFilter(src_array, -1, kernel_shape[::-1], **filter_args)
+        ref_sum = cv.boxFilter(ref_array, -1, kernel_shape[::-1], **filter_args)
+        src_ref_sum = cv.boxFilter(src_array * ref_array, -1, kernel_shape[::-1], **filter_args)
+        mask_sum = cv.boxFilter(mask.astype(RasterArray.default_dtype, copy=False), -1, kernel_shape[::-1], **filter_args)
         m_num_array = (mask_sum * src_ref_sum) - (src_sum * ref_sum)
 
         # find the denominator for the gain i.e. N*var(src)
-        src2_sum = cv.sqrBoxFilter(src_array, -1, kernel_shape, **filter_args)
+        src2_sum = cv.sqrBoxFilter(src_array, -1, kernel_shape[::-1], **filter_args)
         m_den_array = (mask_sum * src2_sum) - (src_sum ** 2)
 
         # create parameter RasterArray filled with nodata
@@ -397,8 +400,8 @@ class KernelModel:
         # Similar to the block overlap amount, this removes ceil(kernel_shape/2) pixels from the nodata edge.  Note,
         # that this is the strict approach for proc_crs == ref, it could be floor(kernel_shape/2) for proc_crs == src,
         # which avoids the additional upsampling step.
-        se = cv2.getStructuringElement(cv2.MORPH_RECT, tuple(self._kernel_shape + 2))
-        mask_ra.array = cv2.erode(mask, se, borderType=cv2.BORDER_CONSTANT, borderValue=0)
+        se = cv.getStructuringElement(cv.MORPH_RECT, tuple(self._kernel_shape[::-1] + 2))
+        mask_ra.array = cv.erode(mask, se, borderType=cv.BORDER_CONSTANT, borderValue=0)
         return mask_ra
 
     def fit(self, ref_ra, src_ra, kernel_shape=None):
@@ -422,7 +425,7 @@ class KernelModel:
         """
         # TODO: include a CRS comparison below i.e. one that is faster that rasterio's current implementation
         if ((ref_ra.transform != src_ra.transform) or (ref_ra.shape != src_ra.shape)):
-            raise ValueError("'ref_ra' and 'src_ra' must have the same transform and shape")
+            raise ValueError("'ref_ra' and 'src_ra' must have the same CRS, transform and shape")
 
         if kernel_shape is None:
             kernel_shape = self._kernel_shape
@@ -454,6 +457,8 @@ class KernelModel:
         out_ra :RasterArray
             Homogenised block in a RasterArray.
         """
+        if ((param_ra.transform != src_ra.transform) or (param_ra.shape != src_ra.shape)):
+            raise ValueError("'param_ra' and 'src_ra' must have the same CRS, transform and shape")
         out_array = (param_ra.array[0] * src_ra.array) + param_ra.array[1]
         out_ra = RasterArray.from_profile(out_array, param_ra.profile)
         return out_ra
@@ -470,19 +475,53 @@ class RefSpaceModel(KernelModel):
     """
 
     def fit(self, ref_ra, src_ra, **kwargs):
+        """
+        Fit sliding kernel models to reference and source blocks.
+
+        Parameters
+        ----------
+        ref_ra : RasterArray
+            Reference data block in a RasterArray.
+        src_ra : RasterArray
+            Source data block in a RasterArray.  The src_ra bounds should that contain those of ref_ra.
+            Note that in the interests of speed, this is assumed and not checked for.
+
+        Returns
+        -------
+        param_ra :RasterArray
+            RasterArray of sliding kernel model parameters. Gains in first band, offsets in the second, and optionally
+            R2 for each kernel model in the third band when param_image==True.
+        """
         # choose resampling method based on whether we are up- or downsampling
-        resampling = self._downsampling if np.prod(src_ra.res) < np.prod(ref_ra.res) else self._upsampling
+        resampling = self._get_resampling(src_ra.res, ref_ra.res)
         # downsample src_ra to reference CRS and grid
         src_ds_ra = src_ra.reproject(**ref_ra.proj_profile, resampling=resampling)
         # call base class fit with reference and source RasterArrays in the reference CRS & grid
         return KernelModel.fit(self, ref_ra, src_ds_ra, kernel_shape=self._kernel_shape)
 
     def apply(self, src_ra, param_ra):
+        """
+        Apply kernel models to a source block.
+
+        Parameters
+        ----------
+        src_ra : RasterArray
+            Source data block in a RasterArray.
+        param_ra :RasterArray
+            RasterArray of sliding kernel model parameters. Gains in first band, offsets in the second.  The bounds
+            of param_ra should contain those of src_ra.  Note that in the interests of speed, this is assumed and not
+            checked for.
+
+        Returns
+        -------
+        out_ra :RasterArray
+            Homogenised block in a RasterArray.
+        """
         # create a parameter RasterArray containing only the first two bands of param_ra
         # (to speed up the re-projection below)
         _param_ra = RasterArray.from_profile(param_ra.array[:2], param_ra.profile)
         # choose resampling method based on whether we are up- or downsampling
-        resampling = self._upsampling if np.prod(src_ra.res) < np.prod(param_ra.res) else self._downsampling
+        resampling = self._get_resampling(param_ra.res, src_ra.res)
         # re-project param_ra to source CRS and grid
         param_us_ra = _param_ra.reproject(**src_ra.proj_profile, resampling=resampling)
 
@@ -510,8 +549,25 @@ class SrcSpaceModel(KernelModel):
     """
 
     def fit(self, ref_ra, src_ra, **kwargs):
+        """
+        Fit sliding kernel models to reference and source blocks.
+
+        Parameters
+        ----------
+        ref_ra : RasterArray
+            Reference data block in a RasterArray.
+        src_ra : RasterArray
+            Source data block in a RasterArray.  The ref_ra bounds should contain those of src_ra.
+            Note that in the interests of speed, this is assumed and not checked for.
+
+        Returns
+        -------
+        param_ra :RasterArray
+            RasterArray of sliding kernel model parameters. Gains in first band, offsets in the second, and optionally
+            R2 for each kernel model in the third band when param_image==True.
+        """
         # choose resampling method based on whether we are up- or downsampling
-        resampling = self._upsampling if np.prod(src_ra.res) < np.prod(ref_ra.res) else self._downsampling
+        resampling = self._get_resampling(ref_ra.res, src_ra.res)
         # upsample ref_ra to the source CRS and grid
         ref_us_ra = ref_ra.reproject(**src_ra.proj_profile, resampling=resampling)
 
