@@ -110,11 +110,11 @@ class RasterArray(transform.TransformMethodsMixin, windows.WindowMethodsMixin):
         ra: RasterArray
             The constructed RasterArray.
         """
-        if not {'crs' and 'transform' and 'nodata'} <= set(profile):
+        if not {'crs', 'transform', 'nodata'} <= set(profile):
             raise ImageProfileError("'profile' should include 'crs', 'transform' and 'nodata' keys")
 
         if array is None:  # create array filled with nodata
-            if not {'width' and 'height' and 'count' and 'dtype'} <= set(profile):
+            if not {'width', 'height', 'count', 'dtype'} <= set(profile):
                 raise ImageProfileError("'profile' should include 'width', 'height', 'count' and 'dtype' keys")
             array_shape = (profile['count'], profile['height'], profile['width'])
             array = np.full(array_shape, fill_value=profile['nodata'], dtype=profile['dtype'])
@@ -168,24 +168,29 @@ class RasterArray(transform.TransformMethodsMixin, windows.WindowMethodsMixin):
         if len(index_list) > 1:
             array = np.full((len(index_list), window.height, window.width), fill_value=nodata, dtype=cls.default_dtype)
             bounded_array = array[(slice(array.shape[0]), *bounded_slices)]  # a bounded view into array
+            rio_dataset.read(out=bounded_array, indexes=index_list, window=bounded_window,
+                             out_dtype=cls.default_dtype, **kwargs)
         else:
             array = np.full((window.height, window.width), fill_value=nodata, dtype=cls.default_dtype)
             bounded_array = array[bounded_slices]  # a bounded view into array
+            rio_dataset.read(out=bounded_array, indexes=index_list[0], window=bounded_window,
+                             out_dtype=cls.default_dtype, **kwargs)
 
         # read into the bounded section of the array
-        rio_dataset.read(out=bounded_array, indexes=indexes, window=bounded_window, out_dtype=cls.default_dtype,
-                         **kwargs)
 
         if is_masked:
             # read the mask from dataset and apply it to the array
             bounded_mask = rio_dataset.dataset_mask(window=bounded_window).astype('bool', copy=False)
-            bounded_array[~bounded_mask] = nodata
+            if bounded_array.ndim == 2:
+                bounded_array[~bounded_mask] = nodata
+            else:
+                bounded_array[:, ~bounded_mask] = nodata
 
         return cls(array, rio_dataset.crs, rio_dataset.transform, nodata=nodata, window=window)
 
     @staticmethod
     def bounded_window_slices(rio_dataset, window):
-        """ Bounded array slices and dataset window from boundless dataset and window """
+        """ Bounded array slices and dataset window from dataset and boundless window """
 
         # find window UL and BR corners and crop to rio_dataset bounds
         win_ul = np.array((window.row_off, window.col_off))
@@ -251,7 +256,7 @@ class RasterArray(transform.TransformMethodsMixin, windows.WindowMethodsMixin):
     @property
     def res(self) -> Tuple[float, float]:
         """Array (x, y) resolution (m)."""
-        return tuple(np.abs((self._transform.a, self._transform.e)))
+        return (self._transform.a, -self._transform.e)
 
     @property
     def bounds(self) -> Tuple[float,]:
@@ -262,7 +267,7 @@ class RasterArray(transform.TransformMethodsMixin, windows.WindowMethodsMixin):
     def profile(self) -> dict:
         """The RasterArray properties formatted as a dictionary, compatible with rasterio."""
         return dict(crs=self._crs, transform=self._transform, nodata=self._nodata, count=self.count,
-                    width=self.width, height=self.height, bounds=self.bounds, dtype=self.dtype)
+                    width=self.width, height=self.height, dtype=self.dtype)
 
     @property
     def proj_profile(self) -> dict:
@@ -378,18 +383,23 @@ class RasterArray(transform.TransformMethodsMixin, windows.WindowMethodsMixin):
         kwargs: optional
                 Arguments to passed through the dataset's write() method.
         """
-        # TODO: include a CRS comparison below i.e. one that is faster that rasterio's current implementation
         if not np.all(self.res == rio_dataset.res):
             raise ImageFormatError(f"The dataset resolution does not match that of the RasterArray. "
                                    f"Dataset res: {rio_dataset.res}, RasterArray res: {self.res}")
+        # TODO: raise an issue with rasterio about the speed of crs comparison.  comparing the _crs attr is faster.
+        if self.crs._crs != rio_dataset.crs._crs:
+            raise ImageFormatError(f"The dataset CRS does not match that of the RasterArray. "
+                                   f"Dataset CRS: {rio_dataset.crs.to_proj4()}, RasterArray CRS: {self.crs.to_proj4()}")
+
         if indexes is None:
             indexes = utils.get_nonalpha_bands(rio_dataset)
+            indexes = indexes if len(indexes) > 1 else indexes[0]
 
-        if np.any(np.array(indexes) > rio_dataset.count):
+        if np.any((np.array(indexes) < 1) | (np.array(indexes) > rio_dataset.count)):
             error_indexes = np.array(indexes)[np.array(indexes) > rio_dataset.count]
-            raise ValueError(f'Band index(es) ({error_indexes}) exceed the dataset band count ({rio_dataset.count})')
+            raise ValueError(f'Band index(es) {error_indexes} are out of the valid range (1..{rio_dataset.count})')
 
-        if isinstance(indexes, list) and (len(indexes) > self.count):
+        if (not np.isscalar(indexes)) and (len(indexes) > self.count):
             raise ValueError(f'The length of indexes ({len(indexes)}) exceeds the number of bands in the '
                              f'RasterArray ({self.count})')
 
@@ -418,7 +428,7 @@ class RasterArray(transform.TransformMethodsMixin, windows.WindowMethodsMixin):
         filename: str, pathlib.Path
                   The name of the file to create.
         driver: str, optional
-                A valid raster short format driver name - see http://www.gdal.org/formats_list.html.
+                A valid rasterio short format driver name - see http://www.gdal.org/formats_list.html.
         kwargs: dict, optional
                 Driver specific creation options e.g. compression='deflate' for a geotiff.
                 See the rasterio documentation for more details:
@@ -438,7 +448,7 @@ class RasterArray(transform.TransformMethodsMixin, windows.WindowMethodsMixin):
              The CRS to project into.  [default: use the CRS of the this RasterArray]
         transform: rasterio.transform.Affine, optional
                     The geo-transform to project into.  If 'transform' is specified, 'shape' is also required.
-                    [default: use the transform of the this RasterArray]
+                    [default: use the transform of this RasterArray]
         shape: tuple, optional
                The (rows, columns) size of the destination array. [default: use the shape of this RasterArray]
         nodata: float, int, optional
@@ -456,15 +466,15 @@ class RasterArray(transform.TransformMethodsMixin, windows.WindowMethodsMixin):
             The reprojected RasterArray.
         """
 
-        if transform and not shape:
+        if transform is not None and shape is None:
             raise ValueError('If "transform" is specified, "shape" must also be specified')
 
         if isinstance(resampling, str):
             resampling = Resampling[resampling]
 
-        crs = crs or self._crs
-        shape = shape or self._array.shape
-        dtype = dtype or self._array.dtype
+        crs = crs or self.crs
+        shape = shape or self.shape
+        dtype = dtype or self.dtype
 
         if self.array.ndim > 2:
             _dst_array = np.zeros((self._array.shape[0], *shape), dtype=dtype)
